options(digits = 4)

library(lme4)
library(glmmML)

dw06_raw = read.csv("/Users/mine/Desktop/SCC/Executive Comp (David Walker)/dw_2006_dataraw_v2.csv", h = T)

dw07_raw = read.csv("/Users/mine/Desktop/SCC/Executive Comp (David Walker)/dw_2007_dataraw_v2.csv", h = T)

# Keep companies that had some variety

dw06a = dw06_raw[dw06_raw$keep_ind == 1,]
dw07a = dw07_raw[dw07_raw$keep_ind == 1,]

# Get rid of unused columns, in order to get rid of NA's properly

dw06b = subset(dw06a, select = c(dw_outcome, tobin_s_q, bs_volatility, tobin_bsvol_int, debt_eq_ratio, sales, assets, mktval, roa, trs1yr, trs3yr, ceo_dummy, cfo_dummy, pre_grant_mix, gindex, eindex, ind1, ind2, ind3, ind4, ind5, ind6, ind7, ind8, ind9, ind10, ind11, ind12, ind13, ind14, ind15, ind16, ind17, ind18, coname))

dw07b = subset(dw07a, select = c(dw_outcome, tobin_s_q, bs_volatility, tobin_bsvol_int, debt_eq_ratio, sales, assets, mktval, roa, trs1yr, trs3yr, ceo_dummy, cfo_dummy, pre_grant_mix, gindex, eindex, ind1, ind2, ind3, ind4, ind5, ind6, ind7, ind8, ind9, ind10, ind11, ind12, ind13, ind14, ind15, ind16, ind17, ind18, coname))

# Get rid of NA's

dw06c = na.omit(dw06b)
dw07c = na.omit(dw07b)

# Get rid of outlier leverage ratios

uf06 = summary(dw06c$debt_eq_ratio)[5]+1.5*IQR(dw06c$debt_eq_ratio)
uf07 = summary(dw07c$debt_eq_ratio)[5]+1.5*IQR(dw07c$debt_eq_ratio)

dw06d = dw06c[dw06c$debt_eq_ratio <= uf06,]
dw07d = dw07c[dw07c$debt_eq_ratio <= uf07,]


#############################################################################
# glmmML package (used in final version of report)
#############################################################################


# 2006 model, all covariates included, using glmmML, QQplot doesn't work

g1ml = glmmML(dw_outcome ~ 1 + tobin_s_q + bs_volatility + tobin_bsvol_int + debt_eq_ratio + sales + assets + mktval + roa + trs1yr + trs3yr + ceo_dummy + cfo_dummy + pre_grant_mix + gindex + eindex + ind1 + ind2 + ind3 + ind4 + ind5 + ind6 + ind7 + ind8 + ind9 + ind10 + ind11 + ind13 + ind14 + ind15 + ind16 + ind17 + ind18, cluster = coname, data = dw06d, family=binomial, na.action = na.omit)

model.func = function(X){
	return(list(X,glmmML(X ~ 1 + tobin_s_q + bs_volatility + tobin_bsvol_int + debt_eq_ratio + sales + assets + mktval + roa + trs1yr + trs3yr + ceo_dummy + cfo_dummy + pre_grant_mix + gindex + eindex + ind1 + ind2 + ind3 + ind4 + ind5 + ind6 + ind7 + ind8 + ind9 + ind10 + ind11 + ind13 + ind14 + ind15 + ind16 + ind17 + ind18, cluster = coname, data = dw06d, family=binomial)))
}

plot.logisticQQ(g1ml, dw06d, model.func, Reps=100)

# 2007 model, all covariates included, using glmmML, QQplot doesn't work

g2ml = glmmML(dw_outcome ~ 1 + tobin_s_q + bs_volatility + tobin_bsvol_int + debt_eq_ratio + sales + assets + mktval + roa + trs1yr + trs3yr + ceo_dummy + cfo_dummy + pre_grant_mix + gindex + eindex + ind1 + ind2 + ind3 + ind4 + ind5 + ind6 + ind7 + ind8 + ind9 + ind10 + ind11 + ind13 + ind14 + ind15 + ind16 + ind17 + ind18, cluster = coname, data = dw07d, family=binomial)

model.func = function(X){
	return(list(X,glmmML(X ~ 1 + tobin_s_q + bs_volatility + tobin_bsvol_int + debt_eq_ratio + sales + assets + mktval + roa + trs1yr + trs3yr + ceo_dummy + cfo_dummy + pre_grant_mix + gindex + eindex + ind1 + ind2 + ind3 + ind4 + ind5 + ind6 + ind7 + ind8 + ind9 + ind10 + ind11 + ind13 + ind14 + ind15 + ind16 + ind17 + ind18, cluster = coname, data = dw07d, family=binomial)))
}

plot.logisticQQ(g2ml, dw07d, model.func, Reps=100)



#############################################################################
# define function plot.logisticQQ (see report for reference)
#############################################################################

plot.logisticQQ = function(model,data,model.func,Reps=100,main='') {
	
	Fitted<-fitted.glmmML(model=model,data=data)

	#calculate the ordered residuals for the model
	Resids<-sort(res.glmmML(model=model,data=data))
	Resids_0<-Resids[Resids<0]
	Resids_1<-Resids[Resids>=0]

	#specify the number of replicates
	

	#simulate Reps data sets
	Sims<-matrix(rbinom(n=length(Fitted)*Reps,size=1,p=Fitted),nrow=length(Fitted),ncol=Reps)

	#fit the model to each simulated data set
	Models<-apply(Sims,MARGIN=2,FUN=model.func)

	#calculate the ordered (interpolated) simulated residuals and the point-wise 95% confidence intervals
	Resids_Sim<-matrix(unlist(lapply(Models,FUN=function(X){TempData<-data;TempData[,"presence"]<-X[[1]];return(sort(res.glmmML(X[[2]],TempData)))})),ncol=Reps)
	Resids_Sim_0<-matrix(apply(Resids_Sim,MARGIN=2,FUN=function(X){quantile(X[X<0],ppoints(Resids_0,a=1))}),ncol=Reps,nrow=length(Resids_0))
	Resids_Sim_1<-matrix(apply(Resids_Sim,MARGIN=2,FUN=function(X){quantile(X[X>=0],ppoints(Resids_1,a=1))}),ncol=Reps,nrow=length(Resids_1))
	Resids_Sim_0_Median<-apply(Resids_Sim_0,MARGIN=1,FUN=median)
	Resids_Sim_1_Median<-apply(Resids_Sim_1,MARGIN=1,FUN=median)
	Resids_Sim_0_Lower<-apply(Resids_Sim_0,MARGIN=1,FUN=function(X){quantile(X,0.025)})
	Resids_Sim_0_Upper<-apply(Resids_Sim_0,MARGIN=1,FUN=function(X){quantile(X,0.975)})
	Resids_Sim_1_Lower<-apply(Resids_Sim_1,MARGIN=1,FUN=function(X){quantile(X,0.025)})
	Resids_Sim_1_Upper<-apply(Resids_Sim_1,MARGIN=1,FUN=function(X){quantile(X,0.975)})
	#plot the qauntile-quantile plot with 95% confidence intervals and 1:1 line
	plot(0,0,xlim=c(-1,1),ylim=c(-1,1),xlab="simulated quantiles",ylab="fitted quantiles",type='n',main=main)
	points(Resids_Sim_0_Median,Resids_0,cex=0.5)
	points(Resids_Sim_1_Median,Resids_1,cex=0.5)
	lines(Resids_Sim_0_Median,Resids_Sim_0_Lower)
	lines(Resids_Sim_0_Median,Resids_Sim_0_Upper)
	lines(Resids_Sim_1_Median,Resids_Sim_1_Lower)
	lines(Resids_Sim_1_Median,Resids_Sim_1_Upper)
	abline(0,1,lty=3)	
}


pres.glmmML<-function(model,data)
#This function outputs a vector of
#the Pearson residuals for a glmmML object
#- "model" is a glmmML object, and "data"
#is the data frame of the data to which "model"
#was fitted. This function is based on
#the glmmML package version 0.65-5 and
#may not work properly in earlier or later
#versions. Please report any bugs or
#errors to Jonathan Rhodes (j.rhodes@uq.edu.au)
{
	Model<-model
	Data<-data

	#get the model frame
	ModelFrame<-model.frame(formula(terms(Model)),data=Data)

	#get the design matrix
	DesignMatrix<-model.matrix(terms(Model),data=ModelFrame)

	#get the linear predictor
	LP<-DesignMatrix %*% as.matrix(Model$coefficients)

	#get the cluster
	Cluster<-Data[,toString(Model$call$cluster)]
	Cluster<-as.factor(Cluster)

	#add the random-effect posterior modes
	for (i in 1:length(levels(Cluster)))
	{
		LP[Cluster == levels(Cluster)[i]] = LP[Cluster == levels(Cluster)[i]] + Model$posterior.modes[i]
	}

	#get the response
	Response<-as.matrix(model.response(ModelFrame))

	if (is.null(Model$call$family) || (Model$call$family==as.name(paste("binomial(link = ",dQuote("logit"),")",sep=""))) || (Model$call$family==as.name("binomial")))
	#binomial response with logit link function
	{
		#get the fitted p values
		P<-exp(LP) / (1 + exp(LP))

		#get the pearson residuals
		if (ncol(Response) == 1)
		#bernoulli data
		{
			PRes<-(Response - P) / sqrt(P * (1 - P))
		}
		else if (ncol(Response) == 2)
		#binomial data
		{
			PRes<-(Response[,1] - (Response[,1] + Response[,2]) * P) / sqrt((Response[,1] + Response[,2]) * P * (1 - P))
		}
		else
		#response matrix the wrong size
		{
			stop("wrong number of response variables")
		}

	}
	else if (Model$call$family==as.name(paste("binomial(link = ",dQuote("cloglog"),")",sep="")))
	#binomial response with complementary log-log link
	{

		#get the fitted p values
		P<-1 - exp(-exp(LP))

		#get the pearson residuals
		if (ncol(Response) == 1)
		#bernoulli data
		{
			PRes<-(Response - P) / sqrt(P * (1 - P))
		}
		else if (ncol(Response) == 2)
		#binomial data
		{
			PRes<-(Response[,1] - (Response[,1] + Response[,2]) * P) / sqrt((Response[,1] + Response[,2]) * P * (1 - P))
		}
		else
		{
			stop("wrong number of response variables")
		}
	}
	else if ((Model$call$family==as.name(paste("poisson(link = ",dQuote("log"),")",sep=""))) || (Model$call$family==as.name("poisson")))
	#poisson response with log link function
	{

		#get the fitted count values
		C<-exp(LP)

		#get the pearson residuals
		PRes<-(Response - C) / sqrt(C)
	}
	else
	#response not binomial or poisson
	{
		stop("incompatible family specified")
	}

	return(as.vector(PRes))
}



fitted.glmmML<-function(model,data)
#This function outputs a vector of
#the fitted values for a glmmML object
#- "model" is a glmmML object, and "data"
#is the data frame of the data to which "model"
#was fitted. This function is based on
#the glmmML package version 0.65-5 and
#may not work properly in earlier or later
#versions. Please report any bugs or
#errors to Jonathan Rhodes (j.rhodes@uq.edu.au)
{
	Model<-model
	Data<-data

	#get the model frame
	ModelFrame<-model.frame(formula(terms(Model)),data=Data)

	#get the design matrix
	DesignMatrix<-model.matrix(terms(Model),data=ModelFrame)

	#get the linear predictor
	LP<-DesignMatrix %*% as.matrix(Model$coefficients)

	#get the cluster
	Cluster<-Data[,toString(Model$call$cluster)]
	Cluster<-as.factor(Cluster)

	#add the random-effect posterior modes
	for (i in 1:length(levels(Cluster)))
	{
		LP[Cluster == levels(Cluster)[i]] = LP[Cluster == levels(Cluster)[i]] + Model$posterior.modes[i]
	}

	#get the response
	Response<-as.matrix(model.response(ModelFrame))

	if (is.null(Model$call$family) || (Model$call$family==as.name(paste("binomial(link = ",dQuote("logit"),")",sep=""))) || (Model$call$family==as.name("binomial")))
	#binomial response with logit link function
	{
		#get the fitted values
		if (ncol(Response) == 1)
		#bernoulli data
		{
			Fit<-exp(LP) / (1 + exp(LP))
		}
		else if (ncol(Response) == 2)
		#binomial data
		{
			Fit<-(Response[,1] + Response[,2]) * exp(LP) / (1 + exp(LP))
		}
		else
		#response matrix the wrong size
		{
			stop("wrong number of response variables")
		}

	}
	else if (Model$call$family==as.name(paste("binomial(link = ",dQuote("cloglog"),")",sep="")))
	#binomial response with complementary log-log link
	{
		#get the fitted values
		if (ncol(Response) == 1)
		#bernoulli data
		{
			Fit<-1 - exp(-exp(LP))
		}
		else if (ncol(Response) == 2)
		#binomial data
		{
			Fit<-(Response[,1] + Response[,2]) * (1 - exp(-exp(LP)))
		}
		else
		{
			stop("wrong number of response variables")
		}
	}
	else if ((Model$call$family==as.name(paste("poisson(link = ",dQuote("log"),")",sep=""))) || (Model$call$family==as.name("poisson")))
	#poisson response with log link function
	{

		#get the fitted count values
		Fit<-exp(LP)
	}
	else
	#response not binomial or poisson
	{
		stop("incompatible family specified")
	}

	return(as.vector(Fit))
}




res.glmmML<-function(model,data)
#This function outputs a vector of
#the ordinary residuals for a glmmML object
#- "model" is a glmmML object, and "data"
#is the data frame of the data to which "model"
#was fitted. This function is based on
#the glmmML package version 0.65-5 and
#may not work properly in earlier or later
#versions. Please report any bugs or
#errors to Jonathan Rhodes (j.rhodes@uq.edu.au)
{
	Model<-model
	Data<-data

	#get the model frame
	ModelFrame<-model.frame(formula(terms(Model)),data=Data)

	#get the design matrix
	DesignMatrix<-model.matrix(terms(Model),data=ModelFrame)

	#get the linear predictor
	LP<-DesignMatrix %*% as.matrix(Model$coefficients)

	#get the cluster
	Cluster<-Data[,toString(Model$call$cluster)]
	Cluster<-as.factor(Cluster)

	#add the random-effect posterior modes
	for (i in 1:length(levels(Cluster)))
	{
		LP[Cluster == levels(Cluster)[i]] = LP[Cluster == levels(Cluster)[i]] + Model$posterior.modes[i]
	}

	#get the response
	Response<-as.matrix(model.response(ModelFrame))
	res.dim = dim(Response)
	
	if( any(tolower(Response)=="yes") ) {
		Response[tolower(Response) == "yes"] = 1
		Response[tolower(Response) == "no"] = 0
		Response = as.numeric(Response)
		dim(Response) = res.dim
	}
	
	if (is.null(Model$call$family) || (Model$call$family==as.name(paste("binomial(link = ",dQuote("logit"),")",sep=""))) || (Model$call$family==as.name("binomial")))
	#binomial response with logit link function
	{
		#get the fitted p values
		P<-exp(LP) / (1 + exp(LP))

		#get the residuals
		if (ncol(Response) == 1)
		#bernoulli data
		{
			Res<-Response - P
		}
		else if (ncol(Response) == 2)
		#binomial data
		{
			Res<-Response[,1] - (Response[,1] + Response[,2]) * P
		}
		else
		#response matrix the wrong size
		{
			stop("wrong number of response variables")
		}

	}
	else if (Model$call$family==as.name(paste("binomial(link = ",dQuote("cloglog"),")",sep="")))
	#binomial response with complementary log-log link
	{

		#get the fitted p values
		P<-1 - exp(-exp(LP))

		#get the residuals
		if (ncol(Response) == 1)
		#bernoulli data
		{
			Res<-Response - P
		}
		else if (ncol(Response) == 2)
		#binomial data
		{
			Res<-Response[,1] - (Response[,1] + Response[,2]) * P
		}
		else
		{
			stop("wrong number of response variables")
		}
	}
	else if ((Model$call$family==as.name(paste("poisson(link = ",dQuote("log"),")",sep=""))) || (Model$call$family==as.name("poisson")))
	#poisson response with log link function
	{

		#get the fitted count values
		C<-exp(LP)

		#get the residuals
		Res<-Response - C
	}
	else
	#response not binomial or poisson
	{
		stop("incompatible family specified")
	}

	return(as.vector(Res))
}


#############################################################################
# lme4 package (used in first version of report)
#############################################################################

# 2006 model (omitted assets & mktval & debt_eq_ratio & sales)  

model1 = glmer(dw_outcome ~ 1 + tobin_s_q + bs_volatility + tobin_bsvol_int + roa + trs1yr + trs3yr + ceo_dummy + cfo_dummy + gindex + eindex + ind1 + ind2 + ind3 + ind4 + ind5 + ind6 + ind7 + ind8 + ind9 + ind10 + ind11 + ind13 + ind14 + ind15 + ind16 + ind17 + ind18 + (1 | coname), data = dw06, family = binomial(), verbose = FALSE)

# 2007 model (omitted assets & mktval)

model2 = glmer(dw_outcome ~ 1 + tobin_s_q + bs_volatility + tobin_bsvol_int + roa + trs1yr + trs3yr + ceo_dummy + cfo_dummy + gindex + eindex + ind1 + ind2 + ind3 + ind4 + ind5 + ind6 + ind7 + ind8 + ind9 + ind10 + ind11 + ind13 + ind14 + ind15 + ind16 + ind17 + ind18 + (1 | coname), data = dw07, family = binomial(), verbose = FALSE)

# 2007 model (omitted assets & debt_eq_ratio & sales)

model3 = glmer(dw_outcome ~ 1 + tobin_s_q + bs_volatility + tobin_bsvol_int + mktval + roa + trs1yr + trs3yr + ceo_dummy + cfo_dummy + gindex + eindex + ind1 + ind2 + ind3 + ind4 + ind5 + ind6 + ind7 + ind8 + ind9 + ind10 + ind11 + ind13 + ind14 + ind15 + ind16 + ind17 + ind18 + (1 | coname), data = dw07, family = binomial(), verbose = FALSE)

# Plots

# 2006
hist(dw06$dw_outcome, main = "Histogram of Outcome, 2006", xlab = "Outcome")
logit_dw_outcome_06 = log(dw06$dw_outcome / (1 - dw06$dw_outcome))
hist(logit_dw_outcome_06, main = "Histogram of Logit of Outcome, 2006", xlab = "Logit of Outcome")
qqnorm(logit_dw_outcome_06, ylim = c(-8,8))

# 2007

hist(dw07$dw_outcome, main = "Histogram of Outcome, 2007", xlab = "Outcome")
logit_dw_outcome_07 = log(dw07$dw_outcome / (1 - dw07$dw_outcome))
hist(logit_dw_outcome_07, main = "Histogram of \n Logit of Outcome, 2007", xlab = "Logit of Outcome")
qqnorm(logit_dw_outcome_07, ylim = c(-8,8))
